'''This module defines Easy Equities data access components.'''
from datetime import datetime
import json
from frostaura.data_access.personal_asset_data_access import IPersonalAssetDataAccess

class EasyEquitiesPersonalAssetDataAccess(IPersonalAssetDataAccess):
    '''EasyEquities-related functionality.'''

    def get_supported_assets(self) -> list:
        '''Get all supported asset names and symbols.'''

        with open('./../../data/easy_equities_us_stocks.json', 'r') as file:
            return json.load(file)

    def get_personal_transactions(self) -> list:
        '''Get all personal transactions made on an EasyEquities account.'''

        return {
            'TSLA': {
                'name': 'Tesla Inc.',
                'symbol': 'TSLA',
                'transactions': [
                    { 'value': 0.0688, 'date': datetime(2022, 7, 28, 0, 0) }
                ]
            },
            'AAPL': {
                'name': 'Apple Inc.',
                'symbol': 'AAPL',
                'transactions': [
                    { 'value': 0.4317, 'date': datetime(2022, 6, 28, 0, 0) }
                ]
            },
            'DDD': {
                'name': '3D Systems Corporation',
                'symbol': 'DDD',
                'transactions': [
                    { 'value': 8.8925, 'date': datetime(2022, 8, 5, 0, 0) }
                ]
            },
            'SBSW': {
                'name': 'Sibanye Stillwater Ltd',
                'symbol': 'SBSW',
                'transactions': [
                    { 'value': 1.1265, 'date': datetime(2022, 8, 5, 0, 0) }
                ]
            },
        }



'''This module defines Yahoo Finance data access components.'''
import yfinance as yf
import pandas as pd
from frostaura.data_access.public_asset_data_access import IPublicAssetDataAccess

class YahooFinanceDataAccess(IPublicAssetDataAccess):
    '''Yahoo Finance public asset-related functionality.'''

    def get_symbol_history(self, symbol: str) -> pd.DataFrame:
        '''Get historical price movements for a given symbol.'''

        ticker = yf.Ticker(symbol)
        hist = ticker.history(period='max')

        return hist



'''This module defines HTTP resources data access component.'''
import requests
from bs4 import BeautifulSoup
from frostaura.data_access.resources_data_access import IResourcesDataAccess

class HtmlResourcesDataAccess(IResourcesDataAccess):
    '''Component to perform HTML resource related actions.'''

    def get_resource(self, path: str) -> bytes:
        '''Get a queryable HTML page via a URL (path).'''

        response: requests.Response = requests.get(url=path, headers={
            'User-Agent': 'PostmanRuntime/7.29.0'
        })
        response_text: str = response.text

        return BeautifulSoup(response_text, 'html.parser')



'''This module defines embedded resources data access component.'''
from importlib import resources
from typing import BinaryIO
from logging import debug, info
from frostaura.data_access.resources_data_access import IResourcesDataAccess

class EmbeddedResourcesDataAccess(IResourcesDataAccess):
    '''Component to perform embedded resource related actions.'''

    def __init__(self, config: dict):
        self.config = config

    def get_resource(self, path: str) -> BinaryIO:
        '''Get a resource as a byte stream that was embedded in a given package.'''

        info(f'Fetching embedded resource "{path}".')

        if not 'package_name' in self.config:
            debug('No value found for "package_name" in config. Setting default "frostaura".')
            self.config['package_name'] = 'frostaura'

        return resources.open_binary(self.config['package_name'], path)
